@@ -76,16 +76,10 @@ static int kwset_exact_matches;
 static char const *
 kwsincr_case (const char *must)
 {
-  const char *buf;
-  size_t n;
-
-  n = strlen (must);
-#if MBS_SUPPORT
-  if (match_icase && MB_CUR_MAX > 1)
-    buf = mbtolower (must, &n);
-  else
-#endif
-    buf = must;
+  size_t n = strlen (must);
+  const char *buf = (match_icase && MB_CUR_MAX > 1
+                     ? mbtolower (must, &n)
+                     : must);
   return kwsincr (kwset, buf, n);
 }
 
@@ -220,7 +214,6 @@ EGexecute (char const *buf, size_t size, size_t *match_size,
   int backref, start, len, best_len;
   struct kwsmatch kwsm;
   size_t i, ret_val;
-#if MBS_SUPPORT
   if (MB_CUR_MAX > 1)
     {
       if (match_icase)
@@ -233,7 +226,6 @@ EGexecute (char const *buf, size_t size, size_t *match_size,
           buf = case_buf;
         }
     }
-#endif /* MBS_SUPPORT */
 
   mb_start = buf;
   buflim = buf + size;
@@ -261,13 +253,14 @@ EGexecute (char const *buf, size_t size, size_t *match_size,
                 --beg;
               if (kwsm.index < kwset_exact_matches)
                 {
-#if MBS_SUPPORT
+                  if (!MBS_SUPPORT)
+                    goto success;
+
                   if (mb_start < beg)
                     mb_start = beg;
                   if (MB_CUR_MAX == 1
                       || !is_mb_middle (&mb_start, match, buflim,
                                         kwsm.size[0]))
-#endif
                     goto success;
                 }
               if (dfaexec (dfa, beg, (char *) end, 0, NULL, &backref) == NULL)
