@@ -1,6 +1,6 @@
 /* mbsupport.h --- Localize determination of whether we have multibyte stuff.
 
-   Copyright (C) 2004-2005, 2007, 2009-2010 Free Software Foundation, Inc.
+   Copyright (C) 2004-2005, 2007, 2009-2011 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,3 +33,8 @@
 #else
 # define MBS_SUPPORT 0
 #endif
+
+#if ! MBS_SUPPORT
+# undef MB_CUR_MAX
+# define MB_CUR_MAX 1
+#endif
